/*******************************************************************************
 * Copyright (c) 2000, 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package dk.eclipse.sablecceditor;

import java.io.PushbackReader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.BadPositionCategoryException;
import org.eclipse.jface.text.DefaultPositionUpdater;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IPositionUpdater;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.texteditor.IDocumentProvider;
import org.eclipse.ui.texteditor.ITextEditor;
import org.eclipse.ui.views.contentoutline.ContentOutlinePage;
import org.sablecc.sablecc.analysis.DepthFirstAdapter;
import org.sablecc.sablecc.lexer.Lexer;
import org.sablecc.sablecc.node.AHelperDef;
import org.sablecc.sablecc.node.AHelpers;
import org.sablecc.sablecc.node.AIgnTokens;
import org.sablecc.sablecc.node.AProd;
import org.sablecc.sablecc.node.AProductions;
import org.sablecc.sablecc.node.ATokenDef;
import org.sablecc.sablecc.node.ATokens;
import org.sablecc.sablecc.node.Start;
import org.sablecc.sablecc.node.TId;
import org.sablecc.sablecc.node.Token;
import org.sablecc.sablecc.node.AAstProd;
import org.sablecc.sablecc.node.AAst;
import org.sablecc.sablecc.parser.Parser;

	

	
	
	class ContentProvider implements ITreeContentProvider {
		private final SableccContentOutlinePage page;
		private IFile file;
		/**
		 * @param SableccContentOutlinePage
		 */
		ContentProvider(SableccContentOutlinePage page, IFile file) {
			this.page = page;
			this.file= file;
		}
		
		public class SableccCodePositioner extends DepthFirstAdapter{
			private IDocument doc;
			public SableccCodePositioner(IDocument document){
				doc=document;
				
			}
			
			public Position getDefaultPosition(Token tok) throws BadLocationException {
				int ret= tok.getPos()+ doc.getLineOffset(tok.getLine()-1);
				return new Position(ret);
			}
						
			/* (non-Javadoc)
			 * @see org.sablecc.sablecc.analysis.DepthFirstAdapter#inAHelpers(org.sablecc.sablecc.node.AHelpers)
			 */
			public void outAHelpers(AHelpers node) {
				Position temp= new Position(0);
				Position helperDefPos= null;
				
				// there is a helper section, so add it to fContent.
				// here we don't do semantic check, so is for other tokens.
				Segment helperSegment= new Segment(SableccContentOutlinePage.HELPERS, 
					new Position(0));
				getContent().add(helperSegment);
				
				List helperDefs= node.getHelperDefs();
				for (int i=0; i< helperDefs.size(); i++){
					AHelperDef aHelperDef= (AHelperDef)helperDefs.get(i);
					try {
						helperDefPos= getDefaultPosition(aHelperDef.getId());
					} catch(BadLocationException ble) {
						helperDefPos= new Position(0);
					}
					Segment aHelperDefSeg= new Segment(aHelperDef.getId().getText(),
							helperDefPos);
					helperSegment.addChild(aHelperDefSeg);
					if(helperDefPos.getOffset() <= temp.getOffset() || temp.getOffset() == 0)
					{ 
						temp= helperDefPos;
					}
				}
				helperSegment.position = temp;
			}
		
	
			/* (non-Javadoc)
			 * @see org.sablecc.sablecc.analysis.DepthFirstAdapter#outATokens(org.sablecc.sablecc.node.ATokens)
			 */
			public void outATokens(ATokens node) {
				Position temp= new Position(0);
				Position tokenDefPos= null;
				
				Segment tokenSegment= new Segment(SableccContentOutlinePage.TOKENS, 
					new Position(0));
				getContent().add(tokenSegment);
				
				List tokenDefs= node.getTokenDefs();
				for (int i=0; i< tokenDefs.size(); i++){
					ATokenDef aTokenDef= (ATokenDef)tokenDefs.get(i);
					try {
						tokenDefPos= getDefaultPosition(aTokenDef.getId());
					} catch(BadLocationException ble) {
						tokenDefPos= new Position(0);
					}
					Segment aTokenDefSeg= new Segment(aTokenDef.getId().getText(),
									tokenDefPos);
					tokenSegment.addChild(aTokenDefSeg); 
					if(tokenDefPos.getOffset() <= temp.getOffset() || temp.getOffset() == 0)
					{ 
						temp = tokenDefPos;
					}
				}
				tokenSegment.position = temp;
			}
	
			/* (non-Javadoc)
			 * @see org.sablecc.sablecc.analysis.DepthFirstAdapter#outAIgnTokens(org.sablecc.sablecc.node.AIgnTokens)
			 */
			public void outAIgnTokens(AIgnTokens node) {
				Position temp= new Position(0);
				Position igntokenDefPos= null;
				
				Segment igtokenSegment= new Segment(SableccContentOutlinePage.IGNRED_TOKENS, 
					new Position(0));
				getContent().add(igtokenSegment);
				
				List igTokens= node.getListId();
				for (int i=0; i< igTokens.size(); i++){
					TId aIgnTokenDef= (TId)igTokens.get(i);
					try {
						igntokenDefPos= getDefaultPosition(aIgnTokenDef);
					} catch(BadLocationException ble) {
						igntokenDefPos= new Position(0);
					}
					Segment aIgnTokenDefSeg= new Segment(aIgnTokenDef.getText(),
							igntokenDefPos);
					igtokenSegment.addChild(aIgnTokenDefSeg); 
					if(igntokenDefPos.getOffset() <= temp.getOffset() || temp.getOffset() == 0)
					{
						temp= igntokenDefPos;
					}
				}
				igtokenSegment.position = temp;
			}
			
			/* (non-Javadoc)
			 * @see org.sablecc.sablecc.analysis.DepthFirstAdapter#outAProductions(org.sablecc.sablecc.node.AProductions)
			 */
			public void outAProductions(AProductions node) {
				Position temp= new Position(0);
				Position productionDefPos= null;
				
				Segment prodSegment= new Segment(SableccContentOutlinePage.PRODUCTIONS, 
					new Position(0));
				getContent().add(prodSegment);
				
				List prodList= node.getProds();
				
				for (int i=0; i<prodList.size(); i++) {
					AProd aProdDef= (AProd) prodList.get(i);
					try {
						productionDefPos= getDefaultPosition(aProdDef.getId());
					} catch(BadLocationException ble) {
						productionDefPos= new Position(0);
					}
					Segment aProdDefSeg= new Segment(aProdDef.getId().getText(),
							productionDefPos);
					prodSegment.addChild(aProdDefSeg); 
					if(productionDefPos.getOffset() <= temp.getOffset() || temp.getOffset() == 0)
					{
						temp = productionDefPos;
					}
				}
				prodSegment.position = temp;				
			}
			
			public void outAAst(AAst node) {
				Position temp = new Position(0);
				Position astDefPos= null;
				
				Segment astSegment= new Segment(SableccContentOutlinePage.AST, 
						new Position(0));
				getContent().add(astSegment);
				
				List prodList= node.getProds();
				
				for (int i=0; i<prodList.size(); i++) {
					AAstProd aAstDef= (AAstProd) prodList.get(i);
					try {
						astDefPos= getDefaultPosition(aAstDef.getId());
					} catch(BadLocationException ble) {
						astDefPos= new Position(0);
					}
					Segment aAstDefSeg= new Segment(aAstDef.getId().getText(),
							astDefPos);
					astSegment.addChild(aAstDefSeg); 
					if(astDefPos.getOffset() <= temp.getOffset() || temp.getOffset() == 0)
					{
						temp = astDefPos;
					}
				}
				astSegment.position = temp;
			}
	
	
		} //class
		
		
	
	
		protected IPositionUpdater fPositionUpdater =
			new DefaultPositionUpdater(SableccContentOutlinePage.SEGMENTS);
		private List fContent = new ArrayList();
	
		protected void parse(IDocument document) {
			try {
				Lexer lexer = new Lexer(
					new PushbackReader(
					new StringReader(document.get()),
							1024));
				Parser parser = new Parser(lexer);
				Start ast = parser.parse();

				ast.apply(new SableccCodePositioner(document));
			} catch (Exception e) {
				SableccEditorPlugin.getDefault().getLog().log(new Status(Status.WARNING, 
							SableccEditorPlugin.getDefault().getDescriptor().getUniqueIdentifier(),
							Status.OK,
							e.getMessage() +" in "+ file.getName(),
							e));

				getContent().add(new String("Content outline is not available."));
				getContent().add(new String("See Error Log for more details."));
			}
	
		}
	
		/*
		 * @see IContentProvider#inputChanged(Viewer, Object, Object)
		 */
		public void inputChanged(
			Viewer viewer,
			Object oldInput,
			Object newInput) {
			if (oldInput != null) {
				IDocument document = this.page.fDocumentProvider.getDocument(oldInput);
				if (document != null) {
					try {
						document.removePositionCategory(SableccContentOutlinePage.SEGMENTS);
						document.removePositionCategory(SableccContentOutlinePage.HELPERS);
						document.removePositionCategory(SableccContentOutlinePage.TOKENS);
						document.removePositionCategory(SableccContentOutlinePage.IGNRED_TOKENS);
						document.removePositionCategory(SableccContentOutlinePage.PRODUCTIONS);
						document.removePositionCategory(SableccContentOutlinePage.AST);
					} catch (BadPositionCategoryException x) {
					}
					document.removePositionUpdater(fPositionUpdater);
				}
			}
	
			fContent.clear();
	
			if (newInput != null) {
				IDocument document = this.page.fDocumentProvider.getDocument(newInput);
				if (document != null) {
					document.addPositionCategory(SableccContentOutlinePage.SEGMENTS);
					document.addPositionCategory(SableccContentOutlinePage.HELPERS);
					document.addPositionCategory(SableccContentOutlinePage.TOKENS);
					document.addPositionCategory(SableccContentOutlinePage.IGNRED_TOKENS);
					document.addPositionCategory(SableccContentOutlinePage.PRODUCTIONS);
					document.addPositionCategory(SableccContentOutlinePage.AST);
					document.addPositionUpdater(fPositionUpdater);
					parse(document);
				}
			}
		}
	
		/*
		 * @see IContentProvider#dispose
		 */
		public void dispose() {
			if (fContent != null) {
				fContent.clear();
				fContent = null;
			}
		}
	
		/*
		 * @see IContentProvider#isDeleted(Object)
		 */
		public boolean isDeleted(Object element) {
			return false;
		}
	
		/*
		 * @see IStructuredContentProvider#getElements(Object)
		 */
		public Object[] getElements(Object element) {
			return fContent.toArray();
		}
	
		/*
		 * @see ITreeContentProvider#hasChildren(Object)
		 */
		public boolean hasChildren(Object element) {
			if (!(element instanceof Segment))
			return false;
			
			return ((Segment)element).details.size()  != 0;
		}
	
		/*
		 * @see ITreeContentProvider#getParent(Object)
		 */
		public Object getParent(Object element) {
			if (element instanceof Segment)
				return ((Segment)element).parent;
			return null;
		}

		/*
		 * @see ITreeContentProvider#getChildren(Object)
		 */
		public Object[] getChildren(Object element) {
			if (element instanceof Segment)
				return ((Segment)element).details.toArray();
			return new Object[0];
		}

		public void setContent(List fContent) {
			this.fContent = fContent;
		}

		public List getContent() {
			return fContent;
		}
	}
/**
 * A content outline page which always represents the content of the
 * connected editor in 10 segments.
 */
public class SableccContentOutlinePage extends ContentOutlinePage {
	/*
	 * fields modified and added:
	 * 1. segment string for 
	 * 			helper definition
	 * 			tokens
	 * 			ignored tokens
	 * 			productions
	 * 			ast
	 */
	
	public final static String SEGMENTS = "__sablecc_segments"; //$NON-NLS-1$
	public final static String TOKENS = "Tokens";
	public final static String IGNRED_TOKENS =	"Ignored tokens";
	public final static String PRODUCTIONS = "Productions";
	public final static String HELPERS= "Helpers";
	public final static String AST= "Abstract syntax tree";
	
	protected Object fInput;
	protected IDocumentProvider fDocumentProvider;

	protected ITextEditor fTextEditor;

	/**
	 * Creates a content outline page using the given provider and the given editor.
	 */
	public SableccContentOutlinePage(
		IDocumentProvider provider,
		ITextEditor editor) {
		super();
		fDocumentProvider = provider;
		fTextEditor = editor;
	}

	/* (non-Javadoc)
	 * Method declared on ContentOutlinePage
	 */
	public void createControl(Composite parent) {
		/**
		 * several things are done here:
		 * 1 get the viewer inside the content outline page.
		 * 2 set contentprovider and labelprovider for this viewer
		 * 3 set selection listener for this viewer.
		 * 4 set input for this viewer --> it will call contentprovider's input changed.
		 * 
		 */
		super.createControl(parent);

		TreeViewer viewer = getTreeViewer();
		viewer.setContentProvider(new ContentProvider(this, ((IFileEditorInput)fTextEditor.getEditorInput()).getFile()));
		viewer.setLabelProvider(new LabelProvider());
		viewer.addSelectionChangedListener(this);

		if (fInput != null)
			viewer.setInput(fInput);
	}
	
	

	/* (non-Javadoc)
	 * Method declared on ContentOutlinePage
	 */
	public void selectionChanged(SelectionChangedEvent event) {

		super.selectionChanged(event);

		ISelection selection = event.getSelection();
		if (selection.isEmpty())
			fTextEditor.resetHighlightRange();
		else {
			Segment segment =
				(Segment) ((IStructuredSelection) selection).getFirstElement();
			int start = segment.position.getOffset();
			int length = segment.position.getLength();
			try {
				fTextEditor.setHighlightRange(start, length, true);
			} catch (IllegalArgumentException x) {
				fTextEditor.resetHighlightRange();
			}
		}
	}

	/**
	 * Sets the input of the outline page
	 */
	public void setInput(Object input) {
		fInput = input;
		update();
	}

	/**
	 * Updates the outline page.
	 */
	public void update() {
		TreeViewer viewer = getTreeViewer();

		if (viewer != null) {
			Control control = viewer.getControl();
			if (control != null && !control.isDisposed()) {
				control.setRedraw(false);
				viewer.setInput(fInput);
				viewer.expandAll();
				control.setRedraw(true);
			}
		}
	}
	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.contentoutline.ContentOutlinePage#getTreeViewer()
	 */
	public TreeViewer getTreeViewer() {
		
		return super.getTreeViewer();
	}

}


