/*
 * Created on 30-Jan-2004
 */
package dk.eclipse.sablecceditor;

import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;
import org.eclipse.core.runtime.IPath;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.core.resources.IFile;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.core.runtime.Status;

import java.io.File;

import org.sablecc.sablecc.SableCC;

/**
 * @author Pieter van der Spek
 * @version 1.0.0
 * 
 * This class handles the event that is generated when the "Generate compiler"-button is pressed
 * It generates the compiler from the input file or gives feedback to the user if the file contains errors.
 */
public class SableccGenComp implements IEditorActionDelegate{
	private TextEditor fTextEditor;
	
	public void setActiveEditor(IAction action, IEditorPart targetEditor) {
		fTextEditor= (TextEditor)targetEditor;
	}
	
	public void selectionChanged(IAction action, ISelection selection){}
	
	public void run(IAction action) {
		if(fTextEditor instanceof SableccEditor) {
			if(fTextEditor.isDirty()) {
				fTextEditor.doSave(null);
			}
			IFile file= ((IFileEditorInput)fTextEditor.getEditorInput()).getFile();
			IPath location= file.getLocation();
			
			String folder= file.getLocation().toString().substring(0,file.getLocation().toString().lastIndexOf('/')+1);
			
			File newDir = new File(folder+"output/");
			newDir.mkdir();
				
			try {
				SableCC.processGrammar(location.toString(), folder+"output/");
				
				MessageDialog.openInformation(null, "Compiling succeeded", "The file "+file.getName()+" has been succesfully compiled.\n The result can be found in "+newDir.getAbsolutePath());
			}
			catch(Exception e){
				SableccEditorPlugin.getDefault().getLog().log(new Status(Status.ERROR, 
						SableccEditorPlugin.getDefault().getDescriptor().getUniqueIdentifier(),
						Status.OK,
						e.getMessage() +" in "+ file.getName(),
						e));
				
				MessageDialog.openInformation(null, "Compiling failed", "An error in " +file.getName()+ " caused the compiler to break down.\nSee the Error Log for more details.");
			}
		}
	}
}
